package bus_apis

import (
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"github.com/go-admin-team/go-admin-core/sdk/api"
	"go-admin/app/admin/models/bus_models"
	"go-admin/app/admin/service/bus_service"
	"go-admin/tools/app"
	"net/http"
)

type ProductApi struct {
	api.Api
}

// ProductList 查询产品列表
// @Summary 查询产品列表
// @Tags 产品管理-V1.0.0
// @Produce json
// @Accept json
// @Param request body bus_models.ProductListReq true "查询产品列表模型"
// @Success 200 {object} bus_models.ProductListResp
// @Router /api/v1/product/list [post]
func (e ProductApi) ProductList(c *gin.Context) {
	s := bus_service.ProductService{}
	var req bus_models.ProductListReq

	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 调用服务层的 GetProductList 函数获取数据
	resp, err := s.GetProductList(c.Request.Context(), req)
	if err != nil {
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	// 返回查询结果
	app.OK(c, resp, "查询成功")
	return
}

// CreateProduct 新建产品
// @Summary 新建产品
// @Tags 产品管理-V1.0.0
// @Produce json
// @Accept json
// @Param request body bus_models.CreateProductReq true "新建产品模型"
// @Success 200 {object} bus_models.CreateProductResp
// @Router /api/v1/product/create [post]
func (e ProductApi) CreateProduct(c *gin.Context) {
	s := bus_service.ProductService{}
	var req bus_models.CreateProductReq

	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 调用服务层的 CreateProduct 函数来处理逻辑
	resp, err := s.CreateProduct(req)
	if err != nil {
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	// 返回创建结果
	app.OK(c, resp, "新增成功")
	return
}

// EditProduct 编辑产品
// @Summary 编辑产品
// @Tags 产品管理-V1.0.0
// @Produce json
// @Accept json
// @Param request body bus_models.EditProductReq true "编辑产品模型"
// @Success 200 {object} app.Response
// @Router /api/v1/product/edit [post]
func (e ProductApi) EditProduct(c *gin.Context) {
	s := bus_service.ProductService{}
	var req bus_models.EditProductReq

	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 调用服务层的 EditProduct 函数来处理逻辑
	err = s.EditProduct(req)
	if err != nil {
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	// 返回创建结果
	app.OK(c, nil, "编辑成功")
	return
}

// DeleteProduct 删除产品
// @Summary 删除产品
// @Tags 产品管理-V1.0.0
// @Produce json
// @Accept json
// @Param request body bus_models.DeleteProductReq true "删除产品模型"
// @Success 200 {object} app.Response
// @Router /api/v1/product/delete [post]
func (e ProductApi) DeleteProduct(c *gin.Context) {
	s := bus_service.ProductService{}
	var req bus_models.DeleteProductReq

	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 调用服务层的 DeleteProduct 函数来处理逻辑
	err = s.DeleteProduct(req)
	if err != nil {
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	// 返回创建结果
	app.OK(c, nil, "删除成功")
	return
}
