package bus_models

import (
	"go-admin/app/admin/models"
)

// BusProduct 产品表
type BusProduct struct {
	models.Model

	ProductCode string  `gorm:"size:32;not null;uniqueIndex" json:"product_code"` // 产品编码，唯一
	ServiceCode string  `gorm:"size:32;not null;index" json:"service_code"`       // 服务编码，广东深圳移动新产品需要
	ProductName string  `gorm:"size:64;not null" json:"product_name"`             // 产品名称
	ProductAttr uint8   `gorm:"not null" json:"product_attr"`                     // 产品属性：1-直连运营商 2-第三方
	ProductType uint8   `gorm:"not null;index" json:"product_type"`               // 产品类型：1-短信 2-话费 3-流量
	Type        uint8   `gorm:"not null" json:"type"`                             // 类型：1-省内 2-国内
	Size        string  `gorm:"size:16;not null" json:"size"`                     // 流量大小
	Price       float64 `gorm:"type:decimal(10,2);not null" json:"price"`         // 标准价格
	Discount    float64 `gorm:"type:decimal(5,2);not null" json:"discount"`       // 折扣
	RetailPrice float64 `gorm:"type:decimal(10,2);not null" json:"retail_price"`  // 建议零售价格
	Description string  `gorm:"size:128" json:"description,omitempty"`            // 产品描述
	Province    string  `gorm:"size:16;not null" json:"province"`                 // 运营商省份
	City        string  `gorm:"size:32" json:"city,omitempty"`                    // 城市
	Platform    uint8   `gorm:"not null;index" json:"platform"`                   // 运营商：1-移动 2-联通 3-电信
}

// ProductListReq 查询产品列表 - 入参
type ProductListReq struct {
	ProductCode string `json:"product_code,omitempty"` // 产品编码（可选）
	ProductName string `json:"product_name,omitempty"` // 产品名称（可选）
	ProductType uint8  `json:"product_type,omitempty"` // 产品类型：1-短信 2-话费 3-流量（可选）
	Platform    uint8  `json:"platform,omitempty"`     // 运营商：1-移动 2-联通 3-电信（可选）
	Province    string `json:"province,omitempty"`     // 省份（可选）
	City        string `json:"city,omitempty"`         // 城市（可选）
	Page        int    `json:"page"`                   // 页码
	PageSize    int    `json:"page_size"`              // 每页大小
}

// ProductListResp 查询产品列表 - 出参
type ProductListResp struct {
	List     []BusProduct `json:"list"`      // 产品列表
	Total    int          `json:"total"`     // 总记录数
	Page     int          `json:"page"`      // 页码
	PageSize int          `json:"page_size"` // 每页大小
}

// CreateProductReq 新建产品 - 入参
type CreateProductReq struct {
	ProductCode string  `json:"product_code" binding:"required"` // 产品编码
	ServiceCode string  `json:"service_code"`                    // 服务编码
	ProductName string  `json:"product_name" binding:"required"` // 产品名称
	ProductAttr uint8   `json:"product_attr"`                    // 产品属性：1-直连运营商 2-第三方
	ProductType uint8   `json:"product_type" binding:"required"` // 产品类型：1-短信 2-话费 3-流量
	Type        uint8   `json:"type"`                            // 类型：1-省内 2-国内
	Size        string  `json:"size"`                            // 流量大小
	Price       float64 `json:"price" binding:"required"`        // 标准价格
	Discount    float64 `json:"discount"`                        // 折扣
	RetailPrice float64 `json:"retail_price"`                    // 建议零售价格
	Description string  `json:"description"`                     // 产品描述
	Province    string  `json:"province" binding:"required"`     // 运营商省份
	City        string  `json:"city,omitempty"`                  // 城市（可选）
	Platform    uint8   `json:"platform" binding:"required"`     // 运营商：1-移动 2-联通 3-电信
}

// CreateProductResp 新建产品 - 出参
type CreateProductResp struct {
	ID uint64 `json:"id"` // 新创建的产品ID
}

// EditProductReq 编辑产品 - 入参
type EditProductReq struct {
	ID          uint64  `json:"id" binding:"required"`           // 产品ID
	ProductCode string  `json:"product_code" binding:"required"` // 产品编码
	ServiceCode string  `json:"service_code"`                    // 服务编码
	ProductName string  `json:"product_name" binding:"required"` // 产品名称
	ProductAttr uint8   `json:"product_attr"`                    // 产品属性：1-直连运营商 2-第三方
	ProductType uint8   `json:"product_type"`                    // 产品类型：1-短信 2-话费 3-流量
	Type        uint8   `json:"type"`                            // 类型：1-省内 2-国内
	Size        string  `json:"size"`                            // 流量大小
	Price       float64 `json:"price"`                           // 标准价格
	Discount    float64 `json:"discount"`                        // 折扣
	RetailPrice float64 `json:"retail_price"`                    // 建议零售价格
	Description string  `json:"description"`                     // 产品描述
	Province    string  `json:"province"`                        // 运营商省份
	City        string  `json:"city,omitempty"`                  // 城市（可选）
	Platform    uint8   `json:"platform"`                        // 运营商：1-移动 2-联通 3-电信
}

// EditProductResp 编辑产品 - 出参
type EditProductResp struct {
	Success bool `json:"success"` // 是否编辑成功
}

// DeleteProductReq 删除产品 - 入参
type DeleteProductReq struct {
	ID uint64 `json:"id" binding:"required"` // 产品ID
}

// DeleteProductResp 删除产品 - 出参
type DeleteProductResp struct {
	Success bool `json:"success"` // 是否删除成功
}
